@@ -58,14 +58,15 @@ sub brace_edges_expand {
     my ($self, $sel_start, $sel_end, $line) = @_; # look only in $line if defined
     $sel_start = $self->GetCurrentPos unless defined $sel_start;
     $sel_end //= $sel_start;
-    $self->GotoPos( $sel_start - 1 );
+    $self->GotoPos( $sel_start);
     $self->SearchAnchor;
     my $npos = $self->SearchPrev( &Wx::wxSTC_FIND_REGEXP, '[([{]');
     return if defined $line and $line != $self->LineFromPosition( $npos );
     my $match = $self->BraceMatch( $npos );
     return if $match < 0;
     return if defined $line and $line != $self->LineFromPosition( $match );
-    ($npos, $match+1); 
+    return if $npos > $sel_end or $match <= $sel_start;
+    ($npos, $match+1);
 }
 
 sub brace_edges_shrink {
@@ -78,7 +79,7 @@ sub brace_edges_shrink {
     return () if $npos < 0 or $npos >= $sel_end;
     my $match = $self->BraceMatch( $npos );
     return if $match < 0 or $match >= $sel_end;
-    ($npos, $match+1); 
+    ($npos, $match+1);
 }
 
 sub loop_edges_expand {
@@ -216,7 +217,7 @@ sub get_prev_block_start {
         last unless $self->GetLine( $line_nr - 1 ) =~ /\S/;
         $line_nr--;
     }
-    $line_nr;    
+    $line_nr;
 }
 sub get_next_block_start {
     my ($self, $pos) = @_;
@@ -277,4 +278,53 @@ sub next_sub {
     $self->SearchNext( &Wx::wxSTC_FIND_REGEXP, '^\s*sub ');
 }
 
+sub smart_up_pos {
+    my ($self, $pos) = @_;
+    $pos = $self->GetCurrentPos unless defined $pos;
+    my $bpos = $self->prev_brace_pos( $pos );
+    return $bpos if $bpos != $pos;
+    my $line_nr = $self->get_prev_block_start( $pos );
+    defined $line_nr ? $self->PositionFromLine( $line_nr ) : $pos;
+}
+
+sub smart_down_pos {
+    my ($self, $pos) = @_;
+    $pos = $self->GetCurrentPos unless defined $pos;
+    my $bpos = $self->next_brace_pos( $pos );
+    return $bpos if $bpos != $pos;
+    my $line_nr = $self->get_next_block_start( $pos );
+    defined $line_nr ? $self->PositionFromLine( $line_nr ) : $pos;
+}
+
+sub prev_brace_pos {
+    my ($self, $pos) = @_;
+    $pos = $self->GetCurrentPos unless defined $pos;
+    my $char_before = $self->GetTextRange( $pos-1, $pos );
+    my $char_after = $self->GetTextRange( $pos, $pos + 1);
+    if ( $char_before eq ')' or $char_before eq '}' or $char_before eq ']' ) {
+        my $mpos = $self->BraceMatch( $pos - 1 );
+        return $mpos if $mpos != &Wx::wxSTC_INVALID_POSITION;
+    } elsif ($char_after eq ')' or $char_after eq '}' or $char_after eq ']'){
+        my $mpos = $self->BraceMatch( $pos );
+        return $mpos + 1 if $mpos != &Wx::wxSTC_INVALID_POSITION;
+    }
+    $pos;
+}
+
+sub next_brace_pos {
+    my ($self, $pos) = @_;
+    $pos = $self->GetCurrentPos unless defined $pos;
+    my $char_before = $self->GetTextRange( $pos-1, $pos );
+    my $char_after = $self->GetTextRange( $pos, $pos + 1);
+    if ( $char_before eq '(' or $char_before eq '{' or $char_before eq '[' ) {
+        my $mpos = $self->BraceMatch( $pos - 1 );
+        return $mpos if $mpos != &Wx::wxSTC_INVALID_POSITION;
+    } elsif ($char_after eq '(' or $char_after eq '{' or $char_after eq '['){
+        my $mpos = $self->BraceMatch( $pos );
+        return $mpos + 1 if $mpos != &Wx::wxSTC_INVALID_POSITION;
+    }
+    $pos;
+}
+
+
 1;
